package bufferings.ktr.wjr.server.service.junit3;

import java.math.BigDecimal;
import java.util.List;
import java.util.logging.Logger;

import junit.framework.Test;
import junit.framework.TestFailure;
import junit.framework.TestResult;
import junit.framework.TestSuite;
import bufferings.ktr.wjr.shared.model.TestCaseModel;
import bufferings.ktr.wjr.shared.model.TestStoreItem.State;

public class TestRunner3 {

  private static final Logger logger =
    Logger.getLogger(TestRunner3.class.getName());

  /** {@inheritDoc} */
  public List<TestCaseModel> runTests(List<TestCaseModel> testCaseModels) {
    logger.fine("runTests");

    ClassLoader cl = Thread.currentThread().getContextClassLoader();

    for (TestCaseModel testCase : testCaseModels) {
      Class<?> clazz;
      try {
        clazz = Class.forName(testCase.getClassName(), false, cl);
      } catch (ClassNotFoundException e) {
        testCase.setState(State.ERROR);
        testCase.setTrace(e.getMessage());
        continue;
      }

      TestResult result = new TestResult();
      Test test = TestSuite.createTest(clazz, testCase.getMethodName());
      long before = System.currentTimeMillis();
      test.run(result);
      Long time = System.currentTimeMillis() - before;
      testCase.setTime(new BigDecimal(time.toString())
        .movePointLeft(3)
        .toPlainString());

      if (result.failureCount() > 0) {
        testCase.setState(State.FAILURE);
        testCase.setTrace(((TestFailure) result.failures().nextElement())
          .trace());
      } else if (result.errorCount() > 0) {
        testCase.setState(State.ERROR);
        testCase.setTrace(((TestFailure) result.errors().nextElement())
          .trace());
      } else {
        testCase.setState(State.SUCCESS);
        testCase.setTrace(null);
      }
    }
    return testCaseModels;
  }
}
